﻿using PizzeriaContracts.BindingModels;
using PizzeriaContracts.BusinessLogicsContracts;
using PizzeriaContracts.SearchModels;
using PizzeriaContracts.StoragesContracts;
using PizzeriaContracts.ViewModels;
using PizzeriaDataModels;
using Microsoft.Extensions.Logging;
using PizzeriaBusinessLogic.MailWorker;
using DocumentFormat.OpenXml.Validation;

namespace PizzeriaBusinessLogic
{
    public class OrderLogic : IOrderLogic
    {
        private readonly ILogger _logger;
        private readonly IOrderStorage _orderStorage;
        private readonly IClientStorage _clientStorage;
        private readonly AbstractMailWorker _mailWorker;
        public OrderLogic(ILogger<OrderLogic> logger, IOrderStorage orderStorage, IClientStorage clientStorage, AbstractMailWorker mailWorker)
        {
            _logger = logger;
            _orderStorage = orderStorage;
            _clientStorage = clientStorage;
            _mailWorker = mailWorker;
        }
        public List<OrderViewModel>? ReadList(OrderSearchModel? model)
        {
            _logger.LogInformation("Id:{Id}", model?.Id);
            var list = model == null ? _orderStorage.GetFullList() : _orderStorage.GetFilteredList(model);
            if (list == null)
            {
                _logger.LogWarning("ReadList return null list");
                return null;
            }
            _logger.LogInformation("ReadList. Count:{Count}", list.Count);
            return list;
        }

        public OrderViewModel? ReadElement(OrderSearchModel model)
        {
            _logger.LogInformation("Id:{Id}", model?.Id);
            OrderViewModel order = _orderStorage.GetElement(model);
            if (order == null)
            {
                _logger.LogWarning("ReadElement return null");
                return null;
            }
            _logger.LogInformation("ReadElement. OrderId:{Id}", order.Id);
            return order;
        }

        public bool CreateOrder(OrderBindingModel model)
        {
            CheckModel(model);

            if (model.Status != OrderStatus.Неизвестен)
            {
                _logger.LogWarning("Insert operation failed. Incorrect Order status");
                return false;
            }

            model.Status = OrderStatus.Принят;

            var result = _orderStorage.Insert(model);
            if(result == null)
            {
                _logger.LogWarning("Insert operation failed");
                return false;
            }

            var client = _clientStorage.GetElement(new ClientSearchModel { Id = model.ClientId });

            _mailWorker.MailSendAsync(new MailSendInfoBindingModel()
            {
                MailAddress = client.Email,
                Subject = $"Создан новый заказ {result.Id}",
                Text = $"Заказ {result.Id} от {result.DateCreate} на сумму {result.Sum:0.00} принят"
            });

            return true;
        }
        public bool UpdateStatus(OrderBindingModel model, OrderStatus newStatus)
        {
            CheckModel(model, false);

            OrderViewModel? order = _orderStorage.GetElement(new OrderSearchModel {
                Id = model.Id
            });

            if(order == null)
            {
                _logger.LogWarning("Status update to " + newStatus.ToString() + " operation failed. Order not found.");
                return false;
            }

            model.Status = order.Status;
            model.ClientId = order.ClientId;
            if(!model.ImplementerId.HasValue) model.ImplementerId = order.ImplementerId;

            if(model.Status + 1 != newStatus)
            {
                _logger.LogWarning("Status update to " + newStatus.ToString() + " operation failed. Incorrect Order status.");
                return false;
            }

            model.Status = newStatus;

            if(newStatus == OrderStatus.Готов)
            {
                model.DateImplement = DateTime.SpecifyKind(DateTime.Now, DateTimeKind.Utc);
            }

            if (_orderStorage.Update(model) == null)
            {
                _logger.LogWarning("Update operation failed");
                return false;
            }

            var client = _clientStorage.GetElement(new ClientSearchModel { Id = model.ClientId });

            _mailWorker.MailSendAsync(new MailSendInfoBindingModel()
            {
                MailAddress = client.Email,
                Subject = $"Изменен статус заказа {model.Id}",
                Text = $"Статус заказа {model.Id} изменен на {newStatus}"
            });

            return true;
        }
        public bool TakeOrderInWork(OrderBindingModel model)
        {
            return UpdateStatus(model, OrderStatus.Выполняется);
        }

        public bool FinishOrder(OrderBindingModel model)
        {
            return UpdateStatus(model, OrderStatus.Готов);
        }

        public bool DeliveryOrder(OrderBindingModel model)
        {
            return UpdateStatus(model, OrderStatus.Выдан);
        }

        private void CheckModel(OrderBindingModel model, bool withParams = true)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            if (!withParams)
            {
                return;
            }
            if (model.Count <= 0)
            {
                throw new ArgumentNullException("Количество пицц должно быть больше 0", nameof(model.Count));
            }
            if (model.Sum <= 0)
            {
                throw new ArgumentNullException("Сумма заказа должна быть больше 0", nameof(model.Sum));
            }
            _logger.LogInformation("Sum:{ Sum}. Id: { Id}", model.Sum, model.Id);
        }

    }
}
